!! *--------   DSL Code Generator, Simpow   ----------------------
!! *
!! *           Name       : EXCITERAC8B
!! *           Explanation: IEEE, Type AC8B Excitation system model  
!! *                        
!! *           DSL Code Generator, release 1.3, 2005-02-10.
!! *           Copyright STRI AB, Sweden.
!! *--------------------------------------------------------------
!! Department : 
!! Designed by: Tina D. Bystl
!! Checked by : 
!! Approved by: 
!! Date       : 04.10.2007 
!! *--------------------------------------------------------------

PROCESS EXCITERAC8B(E1,SE1,E2,SE2,
& VC,TI,TE,KE,
& TA,VRMIN,VRMAX,KA,
& TD,KD,KI,KP,
& UF,UF0)
EXTERNAL E1,SE1,E2,SE2
EXTERNAL VC,TI,TE,KE
EXTERNAL TA,VRMIN,VRMAX,KA
EXTERNAL TD,KD,KI,KP
EXTERNAL UF0
!! End of external declarations.
REAL     E1,SE1,E2,SE2
REAL     V8,INTER_1/*/,INTER_2/*/
REAL     INTER_3/*/,INTER_4/*/,INTER_5/*/
REAL     INTER_6/*/,INTER_7/*/,INTER_8/*/
REAL     VC,TI,TE,KE
REAL     TA,VRMIN,VRMAX,KA
REAL     TD,KD,KI,KP
REAL     REF/*/,V1,UF0,V3
REAL     V2,V4,V5,INTER_9
REAL     INTER_10,V6,VR,V7
REAL     UF
INTEGER  CHECK_OF_LIMITS
!! End of real and integer declarations.
STATE    V8,V3,INTER_10
STATE    VR,UF,CHECK_OF_LIMITS/0/
!! End of state declarations.
   IF (START) THEN
      IF (START00) THEN
!! Control of quadratic saturation parameters.
         IF (E1.LT/0.001/.0) STOP'Saturation parameter E1 is less than zero.'
         IF (E2.LT/0.001/.0) STOP'Saturation parameter E2 is less than zero.'
         IF (SE1.LT/0.001/.0) STOP'Saturation parameter SE1 is less than zero.'
         IF (SE2.LT/0.001/.0) STOP'Saturation parameter SE2 is less than zero.'
         IF (E1.GE/0.001/.E2) STOP'Saturation parameter E1 is greater than',
&         ' or equal to E2.'
         IF (SE1.GE/0.001/.SE2.AND.SE1.NE.0) STOP'Saturation parameter SE1',
&         ' not equal to zero and is greater than or equal to SE2.'
      ENDIF
!! End of control of saturation parameters.
!! Calculation of quadratic saturation variables
!! INTER_7 and INTER_8.
!! A check if the exciter should be modeled unsaturated.
      IF (SE1.EQ/0.001/.0.AND.SE2.EQ/0.001/.0) THEN
         INTER_1=1
         INTER_2=1
         INTER_3=1
         INTER_4=1
         INTER_5=1
         INTER_6=1
         INTER_7=1
         INTER_8=0
      ELSE
         INTER_1=SE1/SE2
         IF (E1*E2.GT/0.001/.0) THEN
            INTER_2=SQRT(E1*E2)
         ELSE
            INTER_2=0
         ENDIF
         IF (E1.GT/0.001/.0) THEN
            INTER_3=SQRT(E1)
         ELSE
            INTER_3=0
         ENDIF
         IF (E2*INTER_1.GT/0.001/.0) THEN
            INTER_4=SQRT(E2*INTER_1)
         ELSE
            INTER_4=0
         ENDIF
         IF (E2.GT/0.001/.0) THEN
            INTER_5=SQRT(E2)
         ELSE
            INTER_5=0
         ENDIF
         IF (E1*INTER_1.GT/0.001/.0) THEN
            INTER_6=SQRT(E1*INTER_1)
         ELSE
            INTER_6=0
         ENDIF
         INTER_7=INTER_2*(INTER_3-INTER_4)/(INTER_5-INTER_6)
         INTER_8=SE2*((INTER_5-INTER_6)**2)/((E2-E1)**2)
         IF (INTER_7.LT.0) THEN
            STOP'Errors in given saturation parameters.',
&            ' The quadratic saturation curve lies in',
&            ' the left half plane.'
         ENDIF
      ENDIF
!! End of calculation of quadratic saturation
!! variables INTER_7 and INTER_8.
   ENDIF
!! End of parameter setting and initiations of EXCITERAC8B.

!! Here starts the dynamic part of process EXCITERAC8B.
!! A signal subtracted to the reference, Reference.
   V1=REF-Vc
!! An integrator with integral time TI and integral constant KI.
   V3: TI*.D/DT.V3=KI*V1
!! Multiplication of two signals.
   V2=KP*V1
!! Multiplication of two signals.
   V4=KD*V1
!! Filtered deriving function s/(1+sTD).
   INTER_9=V4/TD
   INTER_10: INTER_10=INTER_9-TD*.D/DT.INTER_10
   V5=INTER_9-INTER_10
!! Summation of three signals.
   V6=V2+V3+V5
!! A first-order filter of non-wind-up type, KA/(1+sTA).
   IF (VR.GE.VRMAX.AND.
&     (V6*KA-VRMAX).GT.0.AND..NOT.START) THEN
      VR=VRMAX
      PRINT-I'VR is at maximum limit.'
   ELSEIF (VR.LE.VRMIN.AND.
&         (V6*KA-VRMIN).LE.0.AND..NOT.START) THEN
      VR=VRMIN
      PRINT-I'VR is at minimum limit.'
   ELSE
      IF (START00) THEN
         VR=KA*V6
      ELSE
         VR: VR=KA*V6-TA*.D/DT.VR
      ENDIF
      PRINT'VR is within limits.'
   ENDIF
!! Summation of two signals.
   V7=VR-V8
!! First-order filter with filter constants TE and 
!! KE in the denominator.
   IF (START00) THEN
      UF=UF0                                        !UF=V7/KE Tina(Tips fra Lars) 
   ELSE
      UF: UF*KE=V7-TE*.D/DT.UF
   ENDIF
!! Quadratic saturation.
   IF (INTER_8.EQ/0.001/.0.OR.(UF.LE.INTER_7.AND.
&      UF.GE.-INTER_7)) THEN
      V8=0
   ELSEIF (UF.LE.-INTER_7) THEN
      V8=-INTER_8*(UF+INTER_7)**2
   ELSE
      V8=INTER_8*(UF-INTER_7)**2
   ENDIF
!! Initial control of some of the block diagrams.
   IF (START) THEN
!! Checks start conditions by setting REF.
      REF: UF=UF0
!! A check of the filtered deriving function s/(1+sTD).
      IF (TD.LE.0) THEN
         STOP'Time constant TD in block s/(1+sTF) less or equal zero!'
      ENDIF
!! A check of the first-order filter of non-wind-up type, KA/(1+sTA).
      IF (VRMAX.LE.VRMIN) THEN
         STOP'The upper limit is lower than the lower limit.'
      ENDIF
   ENDIF
!! End of initial control of some of the block diagrams.
!! Control of block diagram outputs within given limits.
   IF (.NOT.START.AND.CHECK_OF_LIMITS.EQ.0) THEN
!! A first-order filter of non-wind-up type, KA/(1+sTA). This is a start-up check.
      IF (VR.GE.VRMAX.AND.
&         (V6*KA-VR).GT.0) THEN
         STOP'VR is at maximum limit.'
      ELSEIF (VR.LE.VRMIN.AND.
&         (V6*KA-VR).LT.0) THEN
         STOP'VR is at minimum limit.'
      ENDIF
      CHECK_OF_LIMITS=1
   ENDIF
!! End of control of block diagram outputs within given limits.
END
!! End of EXCITERAC8B.                :-|



!! *--------   DSL Code Generator, Simpow   ----------------------
!! *
!! *           Name       : EXCITERSIN
!! *           Explanation: IEEE, Type AC8B Excitation system model with sinfi controller
!! *                        
!! *           DSL Code Generator, release 1.3, 2005-02-10.
!! *           Copyright STRI AB, Sweden.
!! *--------------------------------------------------------------
!! Department : 
!! Designed by: Tina D. Bystl
!! Checked by : 
!! Approved by: 
!! Date       : 2007.10.22
!! *--------------------------------------------------------------

PROCESS EXCITERSIN(E1,SE1,E2,SE2,
& VC,PE,
& QE,YMIN,YMAX,VMAX,
& KII,TII,VMIN,KG,
& TI,TE,KE,TA,
& VRMIN,VRMAX,KA,TD,
& KD,KI,KP,UF,
& UF0)                                      !removed PE0 and QE0
EXTERNAL E1,SE1,E2,SE2
EXTERNAL VC,PE
EXTERNAL QE,YMIN,YMAX,VMAX
EXTERNAL KII,TII,VMIN,KG
EXTERNAL TI,TE,KE,TA
EXTERNAL VRMIN,VRMAX,KA,TD
EXTERNAL KD,KI,KP,UF0
!! End of external declarations.
REAL     E1,SE1,E2,SE2
REAL     V8,INTER_1/*/,INTER_2/*/
REAL     INTER_3/*/,INTER_4/*/,INTER_5/*/
REAL     INTER_6/*/,INTER_7/*/,INTER_8/*/
REAL     VC,PE
REAL     PE0/*/,QE0/*/                      !added PE0 and QE0                     
REAL     QE,YMIN,YMAX,VMAX
REAL     KII,TII,VMIN,KG
REAL     TI,TE,KE,TA
REAL     VRMIN,VRMAX,KA,TD
REAL     KD,KI,KP,TANX0
REAL     TANX,REF/*/,V1,UF0
REAL     X0,SINX0,X,SINX
REAL     Y1,Y2,Y3,Y4
REAL     Y5,VS,V9,V4
REAL     V5,INTER_9,INTER_10
REAL     V3,V2,V6,VR
REAL     V7,UF
INTEGER  CHECK_OF_LIMITS
!! End of real and integer declarations.
STATE    V8,TANX0,TANX
STATE    X0,X,Y3,INTER_10
STATE    V3,VR,UF,CHECK_OF_LIMITS/0/
   IF (START) THEN
      IF (START00) THEN
         QE0=QE                            !QE0 and PE0-initialized (equal powerflow values for QE and PE)  
         PE0=PE
         UF=UF0                            !initialization of UFO, REF and Y3 to help simpow getting started
         REF=1.
         Y3=0.
!! Control of quadratic saturation parameters.
         IF (E1.LT/0.001/.0) STOP'Saturation parameter E1 is less than zero.'
         IF (E2.LT/0.001/.0) STOP'Saturation parameter E2 is less than zero.'
         IF (SE1.LT/0.001/.0) STOP'Saturation parameter SE1 is less than zero.'
         IF (SE2.LT/0.001/.0) STOP'Saturation parameter SE2 is less than zero.'
         IF (E1.GE/0.001/.E2) STOP'Saturation parameter E1 is greater than',
&         ' or equal to E2.'
         IF (SE1.GE/0.001/.SE2.AND.SE1.NE.0) STOP'Saturation parameter SE1',
&         ' not equal to zero and is greater than or equal to SE2.'
      ENDIF
!! End of control of saturation parameters.
!! Calculation of quadratic saturation variables
!! INTER_7 and INTER_8.
!! A check if the exciter should be modeled unsaturated.
      IF (SE1.EQ/0.001/.0.AND.SE2.EQ/0.001/.0) THEN
         INTER_1=1
         INTER_2=1
         INTER_3=1
         INTER_4=1
         INTER_5=1
         INTER_6=1
         INTER_7=1
         INTER_8=0
      ELSE
         INTER_1=SE1/SE2
         IF (E1*E2.GT/0.001/.0) THEN
            INTER_2=SQRT(E1*E2)
         ELSE
            INTER_2=0
         ENDIF
         IF (E1.GT/0.001/.0) THEN
            INTER_3=SQRT(E1)
         ELSE
            INTER_3=0
         ENDIF
         IF (E2*INTER_1.GT/0.001/.0) THEN
            INTER_4=SQRT(E2*INTER_1)
         ELSE
            INTER_4=0
         ENDIF
         IF (E2.GT/0.001/.0) THEN
            INTER_5=SQRT(E2)
         ELSE
            INTER_5=0
         ENDIF
         IF (E1*INTER_1.GT/0.001/.0) THEN
            INTER_6=SQRT(E1*INTER_1)
         ELSE
            INTER_6=0
         ENDIF
         INTER_7=INTER_2*(INTER_3-INTER_4)/(INTER_5-INTER_6)
         INTER_8=SE2*((INTER_5-INTER_6)**2)/((E2-E1)**2)
         IF (INTER_7.LT.0) THEN
            STOP'Errors in given saturation parameters.',
&            ' The quadratic saturation curve lies in',
&            ' the left half plane.'
         ENDIF
      ENDIF
!! End of calculation of quadratic saturation
!! variables INTER_7 and INTER_8.
   ENDIF
!! End of parameter setting and initiations of EXCITERSIN.

!! Here starts the dynamic part of process EXCITERSIN.
!! Divides a signal with another. Output signal is a real.
   IF (START00) THEN
    TANX0=1
   ELSE
      IF (.NOT.(PE0.GT.0.OR.PE0.LT.0)) THEN
         IF (START) THEN
            PRINT-I'The input signal is equal zero for the ',
&                  'block diagram with the output TANX0!'
         ELSE
            STOP'The input signal is equal zero for the ',
&               'block diagram with the output TANX0!'
         ENDIF
      ENDIF
      TANX0: TANX0*PE0=QE0
   ENDIF
!! Divides a signal with another. Output signal is a real.
   IF (START00) THEN
    TANX=1
   ELSE
      IF (.NOT.(PE.GT.0.OR.PE.LT.0)) THEN
         IF (START) THEN
            PRINT-I'The input signal is equal zero for the ',
&                  'block diagram with the output TANX!'
         ELSE
            STOP'The input signal is equal zero for the ',
&               'block diagram with the output TANX!'
         ENDIF
      ENDIF
      TANX: TANX*PE=QE
   ENDIF
!! A signal subtracted to the reference, Reference.
   V1=REF-Vc
!! Calculates the atan value of TANX0.
   X0: TANX0=SIN(X0)/COS(X0)
!! Calculates the sine value of X0.
!! X0 shall be given in radians.
   SINX0=SIN(X0)
!! Calculates the atan value of TANX.
   X: TANX=SIN(X)/COS(X)
!! Calculates the sine value of X.
!! X shall be given in radians.
   SINX=SIN(X)
!! Summation of two signals.
   Y1=SINX0-SINX
!! Multiplication of two signals.
   Y2=KG*Y1
!! A non-wind-up integrator with integral time TII and
!! integral constant KII.
   IF (Y3.GE.VMAX.AND.Y2.GT.0.AND..NOT.START) THEN              !(Y3.GE.VMIN.AND.Y2.GE.0.AND..NOT.START) THEN
      Y3=VMAX                                                   !Y3=VMIN
      PRINT-I'Y3 is at maximum limit!'
   ELSEIF (Y3.LE.VMIN.AND.Y2.LT.0.AND..NOT.START) THEN          !(Y3.LE.VMAX.AND.Y2.LT.0.AND..NOT.START) THEN
      Y3=VMIN                                                   !Y3=VMAX
      PRINT-I'Y3 is at minimum limit!'
   ELSE
      IF(.NOT.START) Y3: TII*.D/DT.Y3=KII*Y2
      PRINT'Y3 is within limits.'
   ENDIF
!! Summation of two signals.
   Y4=Y2+Y3
!! Limiter, YMIN <= Y5 <= YMAX.
!! Checking the limits of the Limit function.
   IF (YMAX.LT.YMIN) THEN
      STOP'The upper limit is lower than the lower limit.'
   ENDIF
   IF (Y4.GE.YMAX.AND..NOT.START) THEN
      Y5=YMAX
      PRINT-I'Y5 is at maximum limit.'
   ELSEIF (Y4.LE.YMIN.AND..NOT.START) THEN
      Y5=YMIN
      PRINT-I'Y5 is at minimum limit.'
   ELSE
      Y5=Y4
      PRINT'Y5 is within limits.'
   ENDIF
!! Changes the sign of a signal.
   VS=-Y5
!! Summation of two signals.
   V9=VS+V1
!! Multiplication of two signals.
   V4=KD*V9
!! Filtered deriving function s/(1+sTD).
   INTER_9=V4/TD
   INTER_10: INTER_10=INTER_9-TD*.D/DT.INTER_10
   V5=INTER_9-INTER_10
!! An integrator with integral time TI and integral constant KI.
   V3: TI*.D/DT.V3=KI*V9
!! Multiplication of two signals.
   V2=KP*V9
!! Summation of three signals.
   V6=V2+V3+V5
!! A first-order filter of non-wind-up type, KA/(1+sTA).
   IF (VR.GE.VRMAX.AND.
&     (V6*KA-VRMAX).GE.0.AND..NOT.START) THEN
      VR=VRMAX
      PRINT-I'VR is at maximum limit.'
   ELSEIF (VR.LE.VRMIN.AND.
&         (V6*KA-VRMIN).LE.0.AND..NOT.START) THEN
      VR=VRMIN
      PRINT-I'VR is at minimum limit.'
   ELSE
      IF (START00) THEN
         VR=KA*V6
      ELSE
         VR: VR=KA*V6-TA*.D/DT.VR
      ENDIF
      PRINT'VR is within limits.'
   ENDIF
!! Summation of two signals.
   V7=VR-V8
!! First-order filter with filter constants TE and 
!! KE in the denominator.
   IF (START00) THEN
      UF=UF0                                            !UF=V7/KE
   ELSE
      UF: UF*KE=V7-TE*.D/DT.UF
   ENDIF
!! Quadratic saturation.
   IF (INTER_8.EQ/0.001/.0.OR.(UF.LE.INTER_7.AND.
&      UF.GE.-INTER_7)) THEN
      V8=0
   ELSEIF (UF.LE.-INTER_7) THEN
      V8=-INTER_8*(UF+INTER_7)**2
   ELSE
      V8=INTER_8*(UF-INTER_7)**2
   ENDIF
!! Initial control of some of the block diagrams.
   IF (START) THEN
!! Checks start conditions by setting REF.
      REF: UF=UF0
!! A check of the filtered deriving function s/(1+sTD).
      IF (TD.LE.0) THEN
         STOP'Time constant TD in block s/(1+sTF) less or equal zero!'
      ENDIF
!! A check of the first-order filter of non-wind-up type, KA/(1+sTA).
      IF (VRMAX.LE.VRMIN) THEN
         STOP'The upper limit is lower than the lower limit.'
      ENDIF
   ENDIF
!! End of initial control of some of the block diagrams.
!! Control of block diagram outputs within given limits.
   IF (.NOT.START.AND.CHECK_OF_LIMITS.EQ.0) THEN
!! An integrator of non-wind-up type with integral time TII and
!! integral constant KII. This is a start-up check.
      IF (Y3.GE.VMAX.AND.                     !Replaced VMIN with VMAX
&         Y2.GT.0.OR.
&         Y3.GT.VMAX) THEN
         STOP'Y3 is at maximum limit.'
      ELSEIF (Y3.LE.VMIN.AND.                 !Replaced VMAX with VMIN 
&             Y2.LT.0.OR.
&             Y3.LT.VMIN) THEN
         STOP'Y3 is at minimum limit.'
      ENDIF
      IF (Y4.GE.YMAX) THEN
         PRINT-I'Y5 is at maximum limit.'
      ELSEIF (Y4.LE.YMIN) THEN
         PRINT-I'Y5 is at minimum limit.'
      ENDIF
!! A first-order filter of non-wind-up type, KA/(1+sTA). This is a start-up check.
      IF (VR.LE.VRMAX.AND.
&         (V6*KA-VR).LT.0) THEN
!         STOP'VR is at maximum limit.'
      ELSEIF (VR.LE.VRMIN.AND.
&         (V6*KA-VR).LT.0) THEN
         STOP'VR is at minimum limit.'
      ENDIF
      CHECK_OF_LIMITS=1
   ENDIF
!! End of control of block diagram outputs within given limits.
END
!! End of EXCITERSIN.                :-|



!! *--------   DSL Code Generator, Simpow   ----------------------
!! *
!! *           Name       : EXCITERQE
!! *           Explanation: IEEE, Type AC8B Excitation system model  
!! *                        
!! *           DSL Code Generator, release 1.3, 2005-02-10.
!! *           Copyright STRI AB, Sweden.
!! *--------------------------------------------------------------
!! Department : 
!! Designed by: Tina D. Bystl
!! Checked by : 
!! Approved by: 
!! Date       : 04.10.2007 
!! *--------------------------------------------------------------

PROCESS EXCITERQE(E1,SE1,E2,SE2,
& VC,QE,YMIN,
& YMAX,VMAX,KII,TII,
& VMIN,KG,TI,TE,
& KE,TA,VRMIN,VRMAX,
& KA,TD,KD,KI,
& KP,UF,UF0)
EXTERNAL E1,SE1,E2,SE2
EXTERNAL VC,QE,YMIN
EXTERNAL YMAX,VMAX,KII,TII
EXTERNAL VMIN,KG,TI,TE
EXTERNAL KE,TA,VRMIN,VRMAX
EXTERNAL KA,TD,KD,KI
EXTERNAL KP,UF0
!! End of external declarations.
REAL     E1,SE1,E2,SE2
REAL     Q0/*/                               !Tina 16.10.07 (QREGULATOR.dsl) 
REAL     V8,INTER_1/*/,INTER_2/*/
REAL     INTER_3/*/,INTER_4/*/,INTER_5/*/
REAL     INTER_6/*/,INTER_7/*/,INTER_8/*/
REAL     VC,QE,YMIN
REAL     YMAX,VMAX,KII,TII
REAL     VMIN,KG,TI,TE
REAL     KE,TA,VRMIN,VRMAX
REAL     KA,TD,KD,KI
REAL     KP,Y1,Y2,REF/*/
REAL     V1,UF0,Y3,Y4
REAL     Y5,VS,V9,V4
REAL     V5,INTER_9,INTER_10
REAL     V3,V2,V6,VR
REAL     V7,UF
INTEGER  CHECK_OF_LIMITS
!! End of real and integer declarations.
STATE    V8,Y3,INTER_10
STATE    V3,VR,UF,CHECK_OF_LIMITS/0/
   IF (START) THEN
      IF (START00) THEN
         Q0=QE                                !Tina 16.10.07 (QREGULATOR.dsl)
         REF=1.                               !Tips Lars 19.10.2007
         UF=UF0                               !Tips Lars 19.10.2007 
         Y3=0.                                !Tips Lars 19.10.2007
!! Control of quadratic saturation parameters.
         IF (E1.LT/0.001/.0) STOP'Saturation parameter E1 is less than zero.'
         IF (E2.LT/0.001/.0) STOP'Saturation parameter E2 is less than zero.'
         IF (SE1.LT/0.001/.0) STOP'Saturation parameter SE1 is less than zero.'
         IF (SE2.LT/0.001/.0) STOP'Saturation parameter SE2 is less than zero.'
         IF (E1.GE/0.001/.E2) STOP'Saturation parameter E1 is greater than',
&         ' or equal to E2.'
         IF (SE1.GE/0.001/.SE2.AND.SE1.NE.0) STOP'Saturation parameter SE1',
&         ' not equal to zero and is greater than or equal to SE2.'
      ENDIF
     
     !END IF
!! End of control of saturation parameters.
!! Calculation of quadratic saturation variables
!! INTER_7 and INTER_8.
!! A check if the exciter should be modeled unsaturated.
      IF (SE1.EQ/0.001/.0.AND.SE2.EQ/0.001/.0) THEN
         INTER_1=1
         INTER_2=1
         INTER_3=1
         INTER_4=1
         INTER_5=1
         INTER_6=1
         INTER_7=1
         INTER_8=0
      ELSE
         INTER_1=SE1/SE2
         IF (E1*E2.GT/0.001/.0) THEN
            INTER_2=SQRT(E1*E2)
         ELSE
            INTER_2=0
         ENDIF
         IF (E1.GT/0.001/.0) THEN
            INTER_3=SQRT(E1)
         ELSE
            INTER_3=0
         ENDIF
         IF (E2*INTER_1.GT/0.001/.0) THEN
            INTER_4=SQRT(E2*INTER_1)
         ELSE
            INTER_4=0
         ENDIF
         IF (E2.GT/0.001/.0) THEN
            INTER_5=SQRT(E2)
         ELSE
            INTER_5=0
         ENDIF
         IF (E1*INTER_1.GT/0.001/.0) THEN
            INTER_6=SQRT(E1*INTER_1)
         ELSE
            INTER_6=0
         ENDIF
         INTER_7=INTER_2*(INTER_3-INTER_4)/(INTER_5-INTER_6)
         INTER_8=SE2*((INTER_5-INTER_6)**2)/((E2-E1)**2)
         IF (INTER_7.LT.0) THEN
            STOP'Errors in given saturation parameters.',
&            ' The quadratic saturation curve lies in',
&            ' the left half plane.'
         ENDIF
      ENDIF
!! End of calculation of quadratic saturation
!! variables INTER_7 and INTER_8.
   ENDIF
!! End of parameter setting and initiations of EXCITERQE.

!! Here starts the dynamic part of process EXCITERQE.
!! Summation of two signals.
   Y1=Q0-QE
!! Multiplication of two signals.
   Y2=KG*Y1
!! A signal subtracted to the reference, Reference.
   V1=REF-Vc
!! A non-wind-up integrator with integral time TII and
!! integral constant KII.
   IF (Y3.GE.VMAX.AND.Y2.GT.0.AND..NOT.START) THEN     !(Y3.GE.VMIN.AND.Y2.GE.0.AND..NOT.START) THEN
      Y3=VMAX                                           !Y3=VMINTips Lars 18.10.2007
      PRINT-I'Y3 is at maximum limit!'
   ELSEIF (Y3.LE.VMIN.AND.Y2.LT.0.AND..NOT.START) THEN !(Y3.LE.VMAX.AND.Y2.LT.0.AND..NOT.START) THEN
      Y3=VMIN                                           !Y3=VMAXTips Lars 18.10.2007
      PRINT-I'Y3 is at minimum limit!'
   ELSE
      IF(.NOT.START) Y3: TII*.D/DT.Y3=KII*Y2            !Y3: TII*.D/DT.Y3=KII*Y2
     PRINT'Y3 is within limits.'
  ENDIF
!! Summation of two signals.
   Y4=Y2+Y3
!! Limiter, YMIN <= Y5 <= YMAX.
!! Checking the limits of the Limit function.
   IF (YMAX.LT.YMIN) THEN
      STOP'The upper limit is lower than the lower limit.'
   ENDIF
   IF (Y4.GE.YMAX.AND..NOT.START) THEN
      Y5=YMAX
      PRINT-I'Y5 is at maximum limit.'
   ELSEIF (Y4.LE.YMIN.AND..NOT.START) THEN
      Y5=YMIN
      PRINT-I'Y5 is at minimum limit.'
   ELSE
      Y5=Y4
      PRINT'Y5 is within limits.'
   ENDIF
!! Changes the sign of a signal.
   VS=-Y5
!! Summation of two signals.
   V9=VS+V1
!! Multiplication of two signals.
   V4=KD*V9
!! Filtered deriving function s/(1+sTD).
   INTER_9=V4/TD
   INTER_10: INTER_10=INTER_9-TD*.D/DT.INTER_10
   V5=INTER_9-INTER_10
!! An integrator with integral time TI and integral constant KI.
   V3: TI*.D/DT.V3=KI*V9
!! Multiplication of two signals.
   V2=KP*V9
!! Summation of three signals.
   V6=V2+V3+V5
!! A first-order filter of non-wind-up type, KA/(1+sTA).
   IF (VR.GE.VRMAX.AND.
&     (V6*KA-VRMAX).GE.0.AND..NOT.START) THEN
      VR=VRMAX
      PRINT-I'VR is at maximum limit.'
   ELSEIF (VR.LE.VRMIN.AND.
&         (V6*KA-VRMIN).LE.0.AND..NOT.START) THEN
      VR=VRMIN
      PRINT-I'VR is at minimum limit.'
   ELSE
      IF (START00) THEN
         VR=KA*V6
      ELSE
         VR: VR=KA*V6-TA*.D/DT.VR
      ENDIF
      PRINT'VR is within limits.'
   ENDIF
!! Summation of two signals.
   V7=VR-V8
!! First-order filter with filter constants TE and 
!! KE in the denominator.
   IF (START00) THEN
      UF=UF0                                       !UF=V7/KE !Tina 16.10.07
   ELSE
      UF: UF*KE=V7-TE*.D/DT.UF
   ENDIF
!! Quadratic saturation.
   IF (INTER_8.EQ/0.001/.0.OR.(UF.LE.INTER_7.AND.
&      UF.GE.-INTER_7)) THEN
      V8=0
   ELSEIF (UF.LE.-INTER_7) THEN
      V8=-INTER_8*(UF+INTER_7)**2
   ELSE
      V8=INTER_8*(UF-INTER_7)**2
   ENDIF
!! Initial control of some of the block diagrams.
   IF (START) THEN
!! Checks start conditions by setting REF.
      REF: UF=UF0
!! A check of the filtered deriving function s/(1+sTD).
      IF (TD.LE.0) THEN
         STOP'Time constant TD in block s/(1+sTF) less or equal zero!'
      ENDIF
!! A check of the first-order filter of non-wind-up type, KA/(1+sTA).
      IF (VRMAX.LE.VRMIN) THEN
         STOP'The upper limit is lower than the lower limit.'
      ENDIF
   ENDIF
!! End of initial control of some of the block diagrams.
!! Control of block diagram outputs within given limits.
   IF (.NOT.START.AND.CHECK_OF_LIMITS.EQ.0) THEN
!! An integrator of non-wind-up type with integral time TII and
!! integral constant KII. This is a start-up check.
      IF (Y3.GE.VMAX.AND.                                !endret vmin=>vmax
&          Y2.GT.0.OR.                                  
&          Y3.GT.VMAX) THEN                              !endret vmax=>vmin
          STOP'Y3 is at maximum limit.'
      ELSEIF (Y3.LE.VMIN.AND.                            !endret vmax=>vmin
&             Y2.LT.0.OR.
&             Y3.LT.VMIN) THEN                           !endret vmax=>vmin
         STOP'Y3 is at minimum limit.'
      ENDIF
      IF (Y4.GE.YMAX) THEN
         PRINT-I'Y5 is at maximum limit.'
      ELSEIF (Y4.LE.YMIN) THEN
         PRINT-I'Y5 is at minimum limit.'
      ENDIF
!! A first-order filter of non-wind-up type, KA/(1+sTA). This is a start-up check.
      IF (VR.GE.VRMAX.AND.
&         (V6*KA-VR).GT.0) THEN
         STOP'VR is at maximum limit.'
      ELSEIF (VR.LE.VRMIN.AND.
&         (V6*KA-VR).LT.0) THEN
         STOP'VR is at minimum limit.'
      ENDIF
      CHECK_OF_LIMITS=1
   ENDIF
!! End of control of block diagram outputs within given limits.
END
!! End of EXCITERQE.                :-|

PROCESS TRONDSVC1(BUS1,SINPHI0,I,U,B0,TSH)     !COSPHI0
AC BUS1
EXTERNAL SINPHI0,I,U,TSH
REAL SINPHI0,TSH/*/
AC_SIGNAL I,U
REAL B0/*/,B
REAL IR,II,UR,UI,P,Q,FI,SINFI
STATE B,FI
IR=XPRE(I)                                  !Ny
II=XPIM(I)                                  !Ny
UR=XPRE(U)                                  !Ny
UI=XPIM(U)                                  !Ny
P=UR*IR+UI*II                               !Ny
Q=-UR*II+UI*IR
PRINT Q             !Ny mandag 1. mai 2006/TT

        IF (START0) THEN                       
        B=0.
        FI=0.                        
        ENDIF
        FI: P*SIN(FI)=Q*COS(FI)
        SINFI=SIN(FI) 
        IF(START)THEN
        B=0.
        ELSE
        B:sinphi0-sinfi=TSH*.D/DT.B
        ENDIF
        IR=+UPIM(BUS1)*B
        II=-UPRE(BUS1)*B        
        IPRE(BUS1)=IR
        IPIM(BUS1)=II
IF(.NOT.START.AND.EVENT)THEN   
	B0=B
!        PRINT 'Q=',Q
        PRINT B                            !Ny mandag 1. mai 2006 (TT)
ENDIF
END

!! *--------   DSL Code Generator, Simpow   ----------------------
!! *
!! *           Name       : CREG
!! *           Explanation: cos_fi regulator
!! *                        cos_fi regulator
!! *           DSL Code Generator, release 1.3, 2003-03-04.
!! *           Copyright ABB Power Technologies, Sweden.
!! *--------------------------------------------------------------
!! Department : SINTEF
!! Designed by: TT (with a little help ... )
!! Checked by : 
!! Approved by: 
!! Date       : 2005-01-16
!! *--------------------------------------------------------------

PROCESS CREG(BUS1,KF,B0,BMIN,BMAX,KA,
& TI,VRMIN,VRMAX,TF,VFMAX,VFMIN,
& I,U)
AC BUS1
EXTERNAL KF,B0,BMIN,BMAX,KA
EXTERNAL TI,VRMIN,VRMAX,TF,VFMAX,VFMIN,I,U                           !Frt inn I og U
!! End of external declarations.
AC_SIGNAL I,U       
!! End of external declarations.
REAL     KF,B0,BMIN,BMAX,KA
REAL     TI,VRMIN,VRMAX,TF,VFMAX,VFMIN
REAL     V2,V3,V4,V5,B,COSFIREF/*/
REAL     V1
INTEGER  CHECK_OF_LIMITS
REAL IR,II,UR,UI,P,Q,FI,COSFI
!! End of real and integer declarations.
PLOT     B,COSFI
STATE    V2,V3,CHECK_OF_LIMITS/0/,FI
!! End of state declarations.
   IF (START) THEN
ENDIF
IR=XPRE(I)                                  !Ny
II=XPIM(I)                                  !Ny
UR=XPRE(U)                                  !Ny
UI=XPIM(U)                                  !Ny
P=UR*IR+UI*II                               !Ny
Q=-UR*II+UI*IR  
!   B and B0 and consequently BMIN and BMAX are in pu of network base                            
        IF (START0) THEN                       
        B=B0  
        FI=ATAN2(Q,P)                        
        ENDIF        
        FI:P*SIN(FI)=Q*COS(FI)                
        COSFI=COS(FI) 
                        
          
!! End of parameter setting and initiations of CREG.
!! A signal subtracted to the reference COSFIREF, General reference.
   V1=COSFIREF-COSFI
!! Here starts the dynamic part of process CREG.
!! A first-order filter of non-wind-up type, KF/(1+sTF).
   IF (V2.GE.VFMAX.AND.
&     (V1*KF-VFMAX).GE.0.AND..NOT.START) THEN
      V2=VFMAX
      PRINT'V2 is at maximum limit.'
   ELSEIF (V2.LE.VFMIN.AND.
&         (V1*KF-VFMIN).LE.0.AND..NOT.START) THEN
      V2=VFMIN
      PRINT'V2 is at minimum limit.'
   ELSE
      V2: V2=KF*V1-TF*.D/DT.V2
      PRINT'V2 is unlimited.'
   ENDIF
!! An integrator of non-wind-up type with integral time TI.
   IF (V3.GE.VRMAX.AND.
&      V2.GE.0.AND..NOT.START) THEN
      V3=VRMAX
      PRINT'V3 is at maximum limit.'
   ELSEIF (V3.LE.VRMIN.AND.
&      V2.LT.0.AND..NOT.START) THEN
      V3=VRMIN
      PRINT'V3 is at minimum limit.'
   ELSE
      V3: TI*.D/DT.V3=V2
      PRINT'V3 is unlimited.'
   ENDIF
!! Summation of two signals.
   V4=V2+V3
!! Multiplies two signals.
   V5=KA*V4
!! Limiter, BMIN <= B <= BMAX.
!! Checking the limits of the Limit function.
   IF (BMAX.LT.BMIN) THEN
      STOP'The upper limit is lower than the lower limit.'
   ENDIF
   IF (V5.GE.BMAX.AND..NOT.START) THEN
      B=BMAX
      PRINT'B is at maximum limit.'
   ELSEIF (V5.LE.BMIN.AND..NOT.START) THEN
      B=BMIN
      PRINT'B is at minimum limit.'
   ELSE
      B=V5
      PRINT'B is unlimited.'
   ENDIF
   ! Compute the injected current into the node
        IR=+UPIM(BUS1)*B
        II=-UPRE(BUS1)*B        
        IPRE(BUS1)=IR
        IPIM(BUS1)=II   
!! A signal subtracted to the reference COSFIREF, General reference.
   V1=COSFIREF-COSFI
   IF (START) THEN
!! A check of the first-order filter of non-wind-up type, KF/(1+sTF).
      IF (VFMAX.LE.VFMIN) THEN
         STOP'The upper limit is lower than the lower limit.'
      ENDIF
!! Checks start conditions by setting COSFIREF.
      COSFIREF: B=B0
   ENDIF
!! Control of block diagram outputs within given limits.
   IF (.NOT.START.AND.CHECK_OF_LIMITS.EQ.0) THEN
!! A first-order filter of non-wind-up type, KF/(1+sTF). This is a start-up check.
      IF (V2.GE.VFMAX.AND.
&         (V1*KF-V2).GT.0) THEN
         STOP'V2 is at maximum limit.'
      ELSEIF (V2.LE.VFMIN.AND.
&         (V1*KF-V2).LT.0) THEN
         STOP'V2 is at minimum limit.'
      ENDIF
!! An integrator of non-wind-up type with integral time TI.
!! This is a start-up check.
      IF (V3.GE.VRMAX.AND.V2.GT.0.OR.
&      V3.GT.VRMAX) THEN
         STOP'V3 is at maximum limit.'
      ELSEIF (V3.LE.VRMIN.AND.V2.LT.0.OR.
&      V3.LT.VRMIN) THEN
         STOP'V3 is at minimum limit.'
      ENDIF
      IF (V5.GE.BMAX) THEN
         PRINT'B is at maximum limit.'
      ELSEIF (V5.LE.BMIN) THEN
         PRINT'B is at minimum limit.'
      ENDIF
      CHECK_OF_LIMITS=1
   ENDIF
!! End of control of block diagram outputs within given limits.
END
!! End of CREG.                :-)
